SystemOrganization addCategory: #'Phidget-EToys'!

MixedCurveMorph subclass: #PhdETSegmentOfCircle
	instanceVariableNames: 'center radius angleA angleB'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETSegmentOfCircle methodsFor: 'accessing' stamp: 'pt 6/3/2009 12:22'!
angleA

	^ angleA! !

!PhdETSegmentOfCircle methodsFor: 'accessing' stamp: 'pt 6/3/2009 12:26'!
angleA: anAngle
	"must be in degrees and between 0 and 360"
	( 0 <= anAngle and: [ anAngle <= 360 ] ) ifTrue: [
		angleA := anAngle.
		self updateShape ].! !

!PhdETSegmentOfCircle methodsFor: 'accessing' stamp: 'pt 6/3/2009 12:22'!
angleB

	^ angleB! !

!PhdETSegmentOfCircle methodsFor: 'accessing' stamp: 'pt 6/3/2009 12:26'!
angleB: anAngle
	"must be in degrees and between 0 and 360"
	( 0 <= anAngle and: [ anAngle <= 360 ] ) ifTrue: [
		angleB := anAngle.
		self updateShape ].! !

!PhdETSegmentOfCircle methodsFor: 'accessing' stamp: 'pt 6/3/2009 12:28'!
circleCenter

	^ center! !

!PhdETSegmentOfCircle methodsFor: 'accessing' stamp: 'pt 6/3/2009 12:28'!
circleCenter: aPoint

	center := aPoint.
	self updateShape.! !

!PhdETSegmentOfCircle methodsFor: 'initialization' stamp: 'pt 6/3/2009 17:23'!
initialize

	super initialize.
	center := 0@0.
	angleA := 0.
	angleB  := 0.
	radius := 1.
	slopeClamps := #(0 0 nil 0).
	self color: (Color black alpha: 0.5).
	self borderColor: Color transparent.
	self updateShape.! !

!PhdETSegmentOfCircle methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 17:17'!
pointFromAngle: angle
	"Calculates the point of the cicrle belonging to an angle.
	angle must be in degrees (0<=angle<=360)"
	
	| x y |
	x := ( self servoRadians: angle ) cos * radius.
	y := ( self servoRadians: angle ) sin * radius.
	^ x@y! !

!PhdETSegmentOfCircle methodsFor: 'accessing' stamp: 'pt 6/3/2009 12:22'!
radius

	^ radius! !

!PhdETSegmentOfCircle methodsFor: 'accessing' stamp: 'pt 6/3/2009 12:26'!
radius: aNumber

	radius := aNumber.
	self updateShape.! !

!PhdETSegmentOfCircle methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 17:16'!
servoRadians: aFloat
	| f |
	f := aFloat -90.0.
	(f<0) ifTrue: [ f := f + 360.0 ].
	^ f degreesToRadians! !

!PhdETSegmentOfCircle methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 16:39'!
updateShape

	self setVertices: {	center.
								center + (self pointFromAngle: angleA).
								center + (self pointFromAngle: ( angleA + angleB / 2 )).
								center + (self pointFromAngle: angleB) }.! !

!IconicButton methodsFor: '*Phidget-EToys' stamp: 'pt 5/7/2009 11:45'!
initializeWithThumbnail: aThumbnail withLabel: aLabel andColor: aColor andSend: aSelector to: aReceiver 	with: aPhidget
	"Initialize the receiver to show aThumbnail on its face, giving it the label supplied and arranging for it, when the button goes down on it, to obtain a new morph by sending the supplied selector to the supplied receiver"

	| labeledItem nonTranslucent |
	nonTranslucent := aColor asNontranslucentColor.
	labeledItem := AlignmentMorph newColumn.
	labeledItem color: nonTranslucent.
	labeledItem borderWidth: 0.
	labeledItem
		layoutInset: 4@0;
		cellPositioning: #center.
	labeledItem addMorph: aThumbnail.
	labeledItem addMorphBack: (Morph new extent: (4@4)) beTransparent.
	labeledItem addMorphBack: (TextMorph new
		backgroundColor: nonTranslucent;
		contentsAsIs: aLabel;
		beAllFont: Preferences standardEToysFont;
		centered).

	self
		beTransparent;
		labelGraphic: ((labeledItem imageForm: 32 backgroundColor: nonTranslucent forRectangle: labeledItem fullBounds) replaceColor: nonTranslucent withColor: Color transparent);
		borderWidth: 0;
		target: aReceiver;
		actionSelector: #launchPartVia:label:phidget:;
		arguments: {aSelector. aLabel. aPhidget};
		actWhen: #buttonDown.

	self stationarySetup.! !

CircleMorph subclass: #PhdETOnOffStatusMorph
	instanceVariableNames: 'status'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETOnOffStatusMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:32'!
initialize
	super initialize.
	self borderWidth: 2.
	self off.! !

!PhdETOnOffStatusMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:21'!
off
	status := false.
	self borderColor: Color black.
	self color: Color lightGray.! !

!PhdETOnOffStatusMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:31'!
on
	status := true.
	self borderColor: Color yellow muchDarker.
	self color: Color yellow.! !

!PhdETOnOffStatusMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:28'!
setStatus: aBoolean
	aBoolean
		ifTrue: [ self on ]
		ifFalse: [ self off ].! !

CircleMorph subclass: #PhdETServoPositionMorph
	instanceVariableNames: 'min max current minMorph maxMorph currentMorph lineExtension shadowMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETServoPositionMorph methodsFor: 'initialization' stamp: 'pt 6/3/2009 01:28'!
createAndAddCurrentMorph
	currentMorph := self 
		createLineFrom: self position
		to: self position + (1 @ 1)
		color: Color black
		width: 3.
	self addMorph: currentMorph! !

!PhdETServoPositionMorph methodsFor: 'initialization' stamp: 'pt 6/3/2009 01:28'!
createAndAddMaxMorph
	maxMorph := self 
		createLineFrom: self position
		to: self position + (1 @ 1)
		color: Color red
		width: 2.
	self addMorph: maxMorph! !

!PhdETServoPositionMorph methodsFor: 'initialization' stamp: 'pt 6/3/2009 02:06'!
createAndAddMinMorph
	minMorph := self 
		createLineFrom: self position
		to: self position + (1 @ 1)
		color: (Color r: 0 g: 0.3 b: 0)
		width: 2.
	self addMorph: minMorph! !

!PhdETServoPositionMorph methodsFor: 'initialization' stamp: 'pt 6/3/2009 16:29'!
createAndAddShadowMorph
	shadowMorph := ( PhdETSegmentOfCircle new )
							circleCenter: self center;
							radius: self width / 2.
	self addMorph: shadowMorph! !

!PhdETServoPositionMorph methodsFor: 'initialization' stamp: 'pt 6/3/2009 12:29'!
createAndAddSubmorphs
	self
		createAndAddMinMorph;
		createAndAddMaxMorph;
		createAndAddCurrentMorph;
		createAndAddShadowMorph.! !

!PhdETServoPositionMorph methodsFor: 'initialization' stamp: 'pt 6/2/2009 16:56'!
createLineFrom: startPoint to: endPoint color: aColor width: aNumber

	^ PolygonMorph vertices: {startPoint. endPoint}
			color: aColor borderWidth: aNumber borderColor: aColor! !

!PhdETServoPositionMorph methodsFor: 'accessing' stamp: 'pt 6/2/2009 16:01'!
current

	^ current! !

!PhdETServoPositionMorph methodsFor: 'accessing' stamp: 'pt 6/3/2009 03:21'!
current: aFloat
	"The float must pass the following condition: 0 <= aFloat <= 360"

	( ( 0 <= aFloat ) and: [ aFloat <= 360 ] )  ifTrue: [
		current := aFloat.
		self updateCurrentMorph ].! !

!PhdETServoPositionMorph methodsFor: 'initialization' stamp: 'pt 6/3/2009 17:20'!
initialize
	super initialize.
	self
		color: Color white;
		createAndAddSubmorphs.
	lineExtension := 4.
	min := 0.
	max := 180.
	current := 90.
	self updateSubmorphs.! !

!PhdETServoPositionMorph methodsFor: 'accessing' stamp: 'pt 6/3/2009 01:24'!
lineExtension
	"The number of pixel the min/max lines are visible (=extend) beyond the circle."

	^ lineExtension! !

!PhdETServoPositionMorph methodsFor: 'accessing' stamp: 'pt 6/3/2009 01:24'!
lineExtension: aNumber
	"The number of pixel the min/max lines are visible (=extend) beyond the circle."

	lineExtension := aNumber.! !

!PhdETServoPositionMorph methodsFor: 'accessing' stamp: 'pt 6/2/2009 16:01'!
max

	^ max! !

!PhdETServoPositionMorph methodsFor: 'accessing' stamp: 'pt 6/3/2009 01:13'!
max: aFloat
	"The float must pass the following condition: 0.0 <= aFloat <= 360.0"

	( ( 0 <= aFloat ) and: [ aFloat <= 360 ] )  ifTrue: [
		max := aFloat.
		self updateMaxMorph ].! !

!PhdETServoPositionMorph methodsFor: 'accessing' stamp: 'pt 6/2/2009 16:01'!
min

	^ min! !

!PhdETServoPositionMorph methodsFor: 'accessing' stamp: 'pt 6/3/2009 01:13'!
min: aFloat
	"The float must pass the following condition: 0.0 <= aFloat <= 360.0"

	( ( 0 <= aFloat ) and: [ aFloat <= 360 ] )  ifTrue: [
		min := aFloat.
		self updateMinMorph ].! !

!PhdETServoPositionMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 01:42'!
servoRadians: aFloat
	| f |
	f := aFloat -90.0.
	(f<0) ifTrue: [ f := f + 360.0 ].
	^ f degreesToRadians! !

!PhdETServoPositionMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 01:46'!
updateCurrentMorph

	| p r |
	r := self width / 2.0.
	p := ( (self servoRadians: self current) cos * r ) @ ( (self servoRadians: self current) sin * r ).
	currentMorph setVertices: { self center . self center + p }.! !

!PhdETServoPositionMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 17:19'!
updateMaxMorph

	| p1 p2 r x y |
	r := self width / 2.0.
	x := (self servoRadians: self max) cos.
	y := (self servoRadians: self max) sin.
	p1 := ( x * ( r - self lineExtension ) ) @ ( y * ( r - self lineExtension ) ).
	p2 := ( x * ( r + self lineExtension ) ) @ ( y * ( r + self lineExtension ) ).
	maxMorph setVertices: { self center + p1 . self center + p2 }.
	self updateShadowMorph.! !

!PhdETServoPositionMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 17:19'!
updateMinMorph

	| p1 p2 r x y |
	r := self width / 2.0.
	x := (self servoRadians: self min) cos.
	y := (self servoRadians: self min) sin.
	p1 := ( x * ( r - self lineExtension ) ) @ ( y * ( r - self lineExtension ) ).
	p2 := ( x * ( r + self lineExtension ) ) @ ( y * ( r + self lineExtension ) ).
	minMorph setVertices: { self center + p1 . self center + p2 }.
	self updateShadowMorph.! !

!PhdETServoPositionMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 16:28'!
updateShadowMorph

	shadowMorph
		circleCenter: self center;
		radius: self width / 2;
		angleA: max;
		angleB: min.! !

!PhdETServoPositionMorph methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 16:27'!
updateSubmorphs

	self
		updateMinMorph;
		updateMaxMorph;
		updateCurrentMorph;
		updateShadowMorph.! !

BorderedMorph subclass: #PhdETPhidget
	instanceVariableNames: 'phidHandle labelMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETPhidget class methodsFor: 'as yet unclassified' stamp: 'pt 5/28/2009 14:26'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."
	^ #(
		(Phidget (
			(slot Label 'The label of this Phidget.' String readWrite Player getLabel Player setLabel:)
			(slot Attached 'This is wether the Phidget is attached to the USB port or not.' Boolean readOnly Player getAttached unused unused)
		))
	)

! !

!PhdETPhidget methodsFor: 'private' stamp: 'pt 5/28/2009 14:38'!
adaptSize
	"Adjusts submorphs to fit perfectly - even if I was resized."
	
	self positionLabel.! !

!PhdETPhidget methodsFor: 'private' stamp: 'pt 5/28/2009 14:26'!
changed
	super changed.
	self adaptSize.! !

!PhdETPhidget methodsFor: 'initialization' stamp: 'pt 5/28/2009 14:54'!
createAndAddSubmorphs
	
	"create and add label"
	labelMorph := StringMorph new font: (TextStyle default fontAt: 2) emphasis: 1.
	self addMorph: labelMorph.! !

!PhdETPhidget methodsFor: 'submorphs-add/remove' stamp: 'pt 5/28/2009 14:41'!
delete
	phidHandle close.
	super delete.! !

!PhdETPhidget methodsFor: 'accessing' stamp: 'lk 6/15/2009 12:10'!
getAttached
	^ phidHandle isAttached! !

!PhdETPhidget methodsFor: 'accessing' stamp: 'pt 5/28/2009 14:31'!
getLabel
	^ labelMorph contents! !

!PhdETPhidget methodsFor: 'initialization' stamp: 'pt 5/28/2009 14:53'!
initPhidget: aPhidHandle withLabel: aLabel
	
	super initialize.
	phidHandle := aPhidHandle.
	self createAndAddSubmorphs.
	
	"Set standard size and color."
	self
		setLabel: aLabel;
		width: 290;
		height: 75;
		color: (Color r: 0.4 g: 0.7 b: 0.4);
		useRoundedCorners.
	
	"open the phidget, so that it's ready to be used"
	phidHandle openAny.! !

!PhdETPhidget methodsFor: 'private' stamp: 'pt 5/28/2009 14:31'!
positionLabel

	labelMorph position: self center - ( labelMorph width / 2 @ ( self height / 2 - 4 ) ).! !

!PhdETPhidget methodsFor: 'accessing' stamp: 'pt 5/28/2009 14:32'!
setLabel: anObject

	labelMorph contents: anObject asString.
	(self player) ifNotNil: [
		self player setName: labelMorph contents ].
	self adaptSize.
! !

PhdETPhidget subclass: #PhdETPortBasedPhidget
	instanceVariableNames: 'ports bin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

PhdETPortBasedPhidget subclass: #PhdETInterfaceKit
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETInterfaceKit class methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 19:18'!
descriptionForPartsBin
    ^ self partName:    'InterfaceKit'
        categories:     #('Phidgets')
        documentation:  'An Interface Kit'! !

!PhdETInterfaceKit methodsFor: 'initialization' stamp: 'pt 6/3/2009 01:49'!
collectPortDescriptorQuads
	"Lists the Ports I want to show."

	| quads |
	quads := OrderedCollection new.
	PhdETIfKitInputPort addPortDescriptorQuadsTo: quads.
	PhdETIfKitOutputPort addPortDescriptorQuadsTo: quads.
	PhdETIfKitSensorPort addPortDescriptorQuadsTo: quads.
	^ quads! !

!PhdETInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 12:18'!
getInput: index
	^ (phidHandle inputs at: index) value! !

!PhdETInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 12:20'!
getOutput: index
	^ (phidHandle outputs at: index) value! !

!PhdETInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 12:24'!
getSensor: index
	^ (phidHandle sensors at: index) value! !

!PhdETInterfaceKit methodsFor: 'initialization' stamp: 'pt 6/2/2009 10:27'!
initializeToStandAlone
	self initPhidget: PhdCrInterfaceKit new withLabel: 'Interface Kit'.! !

!PhdETInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:12'!
inputPortAvailable: portNumber
	^ phidHandle inputs indexIsInRange: portNumber! !

!PhdETInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:12'!
outputPortAvailable: portNumber
	^ phidHandle outputs indexIsInRange: portNumber! !

!PhdETInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:13'!
sensorPortAvailable: portNumber
	^ phidHandle sensors indexIsInRange: portNumber! !

!PhdETInterfaceKit methodsFor: 'accessing' stamp: 'lk 6/15/2009 12:24'!
setOutput: index to: aBoolean
	^ (phidHandle outputs at: index) value: aBoolean! !

!PhdETPortBasedPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/28/2009 14:38'!
adaptSize
	"Adjusts submorphs to fit perfectly - even if I was resized."
	
	self positionBin.
	self positionLabel.! !

!PhdETPortBasedPhidget methodsFor: 'initialization' stamp: 'pt 5/28/2009 17:09'!
createAndAddSubmorphs
	
	"create and add bin"
	super createAndAddSubmorphs.
	bin := self createBin.
	self addMorph: bin.! !

!PhdETPortBasedPhidget methodsFor: 'initialization' stamp: 'pt 5/28/2009 12:04'!
createBin
	
	| oldResizing aBin |
	aBin := (PhdETPartsBin newPartsBinWithOrientation: #leftToRight from: #())
		listDirection: #leftToRight;
		wrapDirection: #topToBottom;
		setNameTo: 'Ports';
		dropEnabled: false;
		useRoundedCorners;
		controlingMorph: self;
		yourself.

	oldResizing := aBin vResizing.
	aBin
		listDirection: #leftToRight quadList: self collectPortDescriptorQuads;
		vResizing: oldResizing;
		layoutChanged;
		fullBounds.
		
	^ aBin! !

!PhdETPortBasedPhidget methodsFor: 'submorphs-add/remove' stamp: 'pt 5/28/2009 17:09'!
delete
	ports reverseDo: [ :port | port delete ].
	super delete.! !

!PhdETPortBasedPhidget methodsFor: 'initialization' stamp: 'pt 5/28/2009 17:09'!
initPhidget: aPhidHandle withLabel: aLabel
	
	ports := OrderedCollection new.
	super initPhidget: aPhidHandle withLabel: aLabel.	
	bin color: self color lighter lighter.! !

!PhdETPortBasedPhidget methodsFor: 'private' stamp: 'pt 5/28/2009 17:09'!
positionBin
		
	bin
		height: self height - 20;
		top: self top + 17;
		width: self width - 6;
		left: self left + 3.! !

!PhdETPortBasedPhidget methodsFor: 'port handling' stamp: 'pt 5/28/2009 17:10'!
registerPort: aPort
	"If a new Port was draged out of me, it registeres itself here."
	
	ports add: aPort.! !

!PhdETPortBasedPhidget methodsFor: 'port handling' stamp: 'pt 5/28/2009 17:10'!
unregisterPort: aPort
	"If a new Port was draged out of me, it registeres itself here."
	
	ports remove: aPort ifAbsent: [].! !

PhdETPhidget subclass: #PhdETServo1Motor
	instanceVariableNames: 'motorMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETServo1Motor class methodsFor: 'as yet unclassified' stamp: 'pt 6/4/2009 08:53'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."
	^ #(
		#(
			#Phidget
			#(
				#(
					#slot
					#Attached
					'This is wether the Phidget is attached to the USB port or not.'
					#Boolean
					#readOnly
					#Player
					#getAttached
					#unused
					#unused
				)
				#(
					#slot
					#MinPosition
					'This is minimum position the servo motor can handle.'
					#Float
					#readOnly
					#Player
					#getServo1MotorMinPosition
					#unused
					#unused
				)
				#(
					#slot
					#MaxPosition
					'This is maximum position the servo motor can handle.'
					#Float
					#readOnly
					#Player
					#getServo1MotorMaxPosition
					#unused
					#unused
				)
				#(
					#slot
					#Position
					'This is the current position of the servo motor.'
					#Float
					#readWrite
					#Player
					#getServo1MotorCurrentPosition
					#Player
					#setServo1MotorCurrentPosition:
				)
				#(
					#slot
					#PositionInt
					'This is the current position of the servo motor maped as a Number between 0 and 1000.'
					#Number
					#readWrite
					#Player
					#getServo1MotorCurrentPositionNumber
					#Player
					#setServo1MotorCurrentPositionNumber:
				)
				#(
					#slot
					#Engaged
					'This is wether the servo motor is engaged or not.'
					#Boolean
					#readWrite
					#Player
					#getServo1MotorEngaged
					#Player
					#setServo1MotorEngaged:
				)
			)
		)
	)! !

!PhdETServo1Motor class methodsFor: 'as yet unclassified' stamp: 'pt 5/29/2009 09:37'!
descriptionForPartsBin
    ^ self partName:    'Servo 1-Motor'
        categories:     #('Phidgets')
        documentation:  'A Servo Board which connects to 1 servo motor.'! !

!PhdETServo1Motor methodsFor: 'events' stamp: 'pt 6/3/2009 16:30'!
attach
	"Sets motorMorph values for my motorMorph if I was attached."
	self
		setMotorMorphMin;
		setMotorMorphMax;
		setMotorMorphCurrent.! !

!PhdETServo1Motor methodsFor: 'initialization' stamp: 'pt 6/3/2009 02:10'!
createAndAddSubmorphs

	super createAndAddSubmorphs.
	motorMorph := PhdETServoPositionMorph new
									width: 65;
									yourself.
	self addMorph: motorMorph.! !

!PhdETServo1Motor methodsFor: 'events' stamp: 'pt 6/3/2009 02:55'!
detach
	"Sets motorMorph values to zero if I was attached."
	
	motorMorph
		min: (self normalizeDegree: self getMinPosition);
		max: (self normalizeDegree: self getMaxPosition);
		current: (self normalizeDegree: self getCurrentPosition).! !

!PhdETServo1Motor methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:17'!
getCurrentPosition
	
	| curPosition |
	
	(phidHandle motors indexIsInRange: 1) ifFalse: [ ^ 0 ].
	
	curPosition := (phidHandle motors at: 1) position.
	motorMorph current: (self normalizeDegree: curPosition).
	^ curPosition! !

!PhdETServo1Motor methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:24'!
getCurrentPositionNumber
	"Calculates the motor position number relative to the intervall [0, 1000].
	aNumber=0 means that the position is set to minPosition.
	aNumber=1000 means that the position ist set to maxPosition."

	| curPosition max min |
	curPosition := self getCurrentPosition.
	max := self getMaxPosition.
	min := self getMinPosition.
	(min - max) rounded == 0 ifTrue: [ ^ 0 ].
	^ curPosition - min * 1000 / ( max - min ) rounded! !

!PhdETServo1Motor methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:19'!
getEngaged

	(phidHandle motors indexIsInRange: 1) ifFalse: [ ^ false ].
	^ (phidHandle motors at: 1) isEngaged! !

!PhdETServo1Motor methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:17'!
getMaxPosition

	| maxPosition |
	
	(phidHandle motors indexIsInRange: 1) ifFalse: [ ^ 0 ].
	
	maxPosition := (phidHandle motors at: 1) positionMax.
	motorMorph max: (self normalizeDegree: maxPosition).
	^ maxPosition! !

!PhdETServo1Motor methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:16'!
getMinPosition

	| minPosition |
	
	(phidHandle motors indexIsInRange: 1) ifFalse: [ ^ 0 ].
	
	minPosition := (phidHandle motors at: 1) positionMin.
	motorMorph min: (self normalizeDegree: minPosition).
	^ minPosition! !

!PhdETServo1Motor methodsFor: 'initialization' stamp: 'pt 6/3/2009 02:28'!
initialize
	self 
		initPhidget: PhdCrServo new
		withLabel: 'Servo Board'.
	self
		width: 110;
		height: 83;
		initializeMotorMorph.! !

!PhdETServo1Motor methodsFor: 'initialization' stamp: 'lk 6/15/2009 12:06'!
initializeMotorMorph

	phidHandle
		on: #Attach do: [ self attach ];
		on: #Detach do: [ self detach ];
		on: #PositionChange do: [ self positionChange ].

	self
		setMotorMorphMin;
		setMotorMorphMax;
		setMotorMorphCurrent;
		positionMotorMorph.! !

!PhdETServo1Motor methodsFor: 'initialization' stamp: 'pt 6/3/2009 01:47'!
initializeToStandAlone
	
	self initialize.! !

!PhdETServo1Motor methodsFor: 'initialization' stamp: 'pt 6/3/2009 03:16'!
normalizeDegree: degree
	"Brings a float into the interval [0;360]"
	
	| deg |
	( 0 <= degree and: [ degree <= 360 ] ) ifTrue: [ deg := degree ].
	degree < 0 ifTrue: [ deg := degree + 360 ].
	360 < degree ifTrue: [ deg := degree - 360 ].
	^ deg! !

!PhdETServo1Motor methodsFor: 'events' stamp: 'pt 6/3/2009 17:26'!
positionChange
	"Sets motorMorph position to the current position if I it changed."
	
	self setMotorMorphCurrent.! !

!PhdETServo1Motor methodsFor: 'initialization' stamp: 'pt 6/3/2009 02:11'!
positionMotorMorph

	motorMorph position: self center - ((motorMorph width / 2) @ (self height / 2 - 30))! !

!PhdETServo1Motor methodsFor: 'accessing' stamp: 'lk 6/15/2009 21:55'!
setCurrentPosition: aFloat

	motorMorph current: (self normalizeDegree: aFloat).
	^ (phidHandle motors at: 1) position: aFloat! !

!PhdETServo1Motor methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:26'!
setCurrentPositionNumber: aNumber
	"Sets the motor position to a value relative to aNumber.
	aNumber=0 means that the position is set to minPosition.
	aNumber=1000 means that the position ist set to maxPosition."

	| max min |
	( 0 <= aNumber and: [ aNumber <= 1000 ] ) ifFalse: [ ^ self ].
	max := self getMaxPosition.
	min := self getMinPosition.
	min == -1000 ifFalse: [
		self setCurrentPosition: max - min * aNumber / 1000.0 + min ].! !

!PhdETServo1Motor methodsFor: 'accessing' stamp: 'lk 6/16/2009 12:01'!
setEngaged: aBoolean

	^ (phidHandle motors at: 1) setEngaged: aBoolean! !

!PhdETServo1Motor methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 02:51'!
setMotorMorphCurrent

	motorMorph current: (self normalizeDegree: self getCurrentPosition).! !

!PhdETServo1Motor methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 02:50'!
setMotorMorphMax

	motorMorph max: (self normalizeDegree: self getMaxPosition).! !

!PhdETServo1Motor methodsFor: 'as yet unclassified' stamp: 'pt 6/3/2009 02:50'!
setMotorMorphMin

	motorMorph min: (self normalizeDegree: self getMinPosition).! !

BorderedMorph subclass: #PhdETPort
	instanceVariableNames: 'phidget portNumber numberString wire'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

PhdETPort subclass: #PhdETIfKitInputPort
	instanceVariableNames: 'statusMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETIfKitInputPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/8/2009 08:57'!
additionsToViewerCategoryInputPort
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."
	
	^ #(
		InputPort (
			(slot Value 'The value at the digital Intput port on an Interface Kit.' Boolean readOnly Player getIfKitInputValue unused unused)
			(slot PortNumber 'The index of this digital Input port.' Number readWrite Player getPortNumber Player setPortNumber:)
			(slot ShowWire 'This is wether a wire from this port to its Phidget is shown..' Boolean readWrite Player getShowWire Player setShowWire:)
		)
	)! !

!PhdETIfKitInputPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:32'!
portDescription

	^  self
		partName:		'Digital Input'
		categories:		#('Interface Kit')
		documentation:	'An port to a digital input of an Interface Kit. This port is bound to a port of the Interface Kit hardware. With me it is possible to read wether the port returns true or false.'.! !

!PhdETIfKitInputPort methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:29'!
getIfKitInputValue
	| value |
	value := phidget getInput: portNumber.
	statusMorph setStatus: value.
	^ value! !

!PhdETIfKitInputPort methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:26'!
initialize
	super initialize.
	self color: Color red lighter.
	self height: 70.
	statusMorph := PhdETOnOffStatusMorph new.
	self addMorph: statusMorph.
	statusMorph
		position: ( self width // 2 - ( statusMorph width // 2 ) ) @ ( self bottom - statusMorph height - 5 ).! !

!PhdETIfKitInputPort methodsFor: 'as yet unclassified' stamp: 'pt 6/2/2009 10:34'!
isInRange: aNumber
	
	^ phidget sensorPortAvailable: aNumber! !

PhdETPort subclass: #PhdETIfKitOutputPort
	instanceVariableNames: 'statusMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETIfKitOutputPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/8/2009 08:58'!
additionsToViewerCategoryOutputPort
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
		OutputPort (
			(slot Value 'The value at the digital Output port on an Interface Kit.' Boolean readWrite Player getIfKitOutputValue Player setIfKitOutputValue:)
			(slot PortNumber 'The index of this digital Output port.' Number readWrite Player getPortNumber Player setPortNumber:)
			(slot ShowWire 'This is wether a wire from this port to its Phidget is shown..' Boolean readWrite Player getShowWire Player setShowWire:)
		)
	)! !

!PhdETIfKitOutputPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:32'!
portDescription

	^  self
		partName:		'Digital Output'
		categories:		#('Interface Kit')
		documentation:	'An port to a digital output of an Interface Kit. This port is bound to a port of the Interface Kit hardware. With me it is possible to read wether the port returns true or false or set this value'.! !

!PhdETIfKitOutputPort methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:30'!
getIfKitOutputValue
	| value |
	value := phidget getOutput: portNumber.
	statusMorph setStatus: value.
	^ value! !

!PhdETIfKitOutputPort methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:27'!
initialize
	super initialize.
	self color: Color blue muchLighter.
	self height: 70.
	statusMorph := PhdETOnOffStatusMorph new.
	self addMorph: statusMorph.
	statusMorph
		position: ( self width // 2 - ( statusMorph width // 2 ) ) @ ( self bottom - statusMorph height - 5 ).! !

!PhdETIfKitOutputPort methodsFor: 'as yet unclassified' stamp: 'pt 6/2/2009 10:35'!
isInRange: aNumber
	
	^ phidget outputPortAvailable: aNumber! !

!PhdETIfKitOutputPort methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:30'!
setIfKitOutputValue: aBoolean
	statusMorph setStatus: aBoolean.
	^ phidget setOutput: portNumber to: aBoolean.! !

PhdETPort subclass: #PhdETIfKitSensorPort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETIfKitSensorPort class methodsFor: 'as yet unclassified' stamp: 'pt 6/4/2009 08:12'!
additionsToViewerCategorySensorPort
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
		SensorPort (
			(slot Value 'The value at the analog Sensor port on an Interface Kit.' Number readOnly Player getIfKitSensorValue unused unused)
			(slot SensorPortNumber 'The index of this analog Sensor port.' Number readWrite Player getPortNumber Player setPortNumber:)
			(slot ShowWire 'This is wether a wire from this port to its Phidget is shown..' Boolean readWrite Player getShowWire Player setShowWire:)
		)
	)! !

!PhdETIfKitSensorPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:32'!
portDescription

	^  self
		partName:		'Analog Sensor'
		categories:		#('Interface Kit')
		documentation:	'An port to an analog sensor of an Interface Kit. This port is bound to a port of the Interface Kit hardware. With me it is possible to read which value an attached analog sensor returns. This value is between 0 and 1000.'.! !

!PhdETIfKitSensorPort methodsFor: 'as yet unclassified' stamp: 'pt 6/2/2009 10:35'!
getIfKitSensorValue
	^ phidget getSensor: portNumber! !

!PhdETIfKitSensorPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:39'!
initialize
	super initialize.
	self color: Color yellow lighter.! !

!PhdETIfKitSensorPort methodsFor: 'as yet unclassified' stamp: 'pt 6/2/2009 10:35'!
isInRange: aNumber
	
	^ phidget inputPortAvailable: aNumber! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:17'!
addPortDescriptorQuadsTo: aList
	"analogous to addPartsDescriptorQuadsTo: aList if: aBlock from the Morph class"
	
	| info |
	info := self portDescription.
	aList add:
		{info globalReceiverSymbol.
		info nativitySelector.
		info formalName.
		info documentation.
		info sampleImageFormOrNil}.! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:00'!
launchPartVia: aSelector label: aString phidget: aPhidget
	"Obtain a morph by sending aSelector to self, and attach it to the morphic hand.  This provides a general protocol for parts bins"

	| aMorph |
	aMorph := self perform: aSelector with: aPhidget.
	aMorph setNameTo: (ActiveWorld unusedMorphNameLike: aString).
	aMorph setProperty: #beFullyVisibleAfterDrop toValue: true.
	aMorph openInHand! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 10:54'!
newStandAlone: aPhidget
	"Answer an instance capable of standing by itself as a usable morph."

	^ self basicNew initializeToStandAlone: aPhidget! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 10:52'!
partName: aName categories: aList documentation: aDoc
	"Answer a DescriptionForPartsBin which will represent a launch of a new instance of my class via the #newStandAlone protocol sent to my class. Use the category-list and documentation provided"


	^ DescriptionForPartsBin new
		formalName: aName
		categoryList: aList
		documentation: aDoc
		globalReceiverSymbol: self name
		nativitySelector: #newStandAlone:! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:18'!
portDescription

	self subclassResponsibility.
	! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:17'!
centerNumberString
	| stringCenter topMiddle |
	topMiddle := (self width // 2)@15.
	stringCenter := (numberString width / 2)@( numberString height / 2).
	numberString position: topMiddle - stringCenter.! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 6/2/2009 10:34'!
createWire
	wire := LineMorph new.
	self currentWorld addMorphBack: wire! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 6/2/2009 10:34'!
delete
	wire delete.
	phidget unregisterPort: self.
	super delete.! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 09:28'!
getPortNumber
	^ portNumber! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 6/10/2009 08:12'!
initialize
	super initialize.
	self useRoundedCorners.
	portNumber := 1.
	numberString := (StringMorph new
							contents: portNumber asString;
							emphasis: 1;
							yourself).
	self
		centerNumberString;
		addMorph: numberString.! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 6/2/2009 10:34'!
initializeToStandAlone: aPhidget
	phidget := aPhidget.
	phidget registerPort: self.
	self initialize.
	! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 19:05'!
isInRange: aNumber
	"A subclass may override this, to allow changing portNumbers"
	^ false! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 6/2/2009 10:34'!
positionWire
	"Positions the Wire so that itreaches from the origin Phidgits to me. If the Wire doenstn exists, create it."

	"Cannot create Wire while initialisation, because that ruins my Thumbnail. So I do it here."
	wire ifNil: [ self createWire ].
	wire setVertices: {phidget center. self center}.! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 19:11'!
setPortNumber: aNumber
	( aNumber isInteger and: [ self isInRange: aNumber ] )
	ifTrue: [
		portNumber := aNumber.
		numberString contents: portNumber asString.
		self centerNumberString ]! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/8/2009 09:23'!
step
	self positionWire.! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/8/2009 09:22'!
stepTime
	^ 100! !

PartsBin subclass: #PhdETPartsBin
	instanceVariableNames: 'controlingMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETPartsBin class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 13:17'!
thumbnailForQuad: aQuint color: aColor phidget: aPhidget
	"Answer a thumbnail for a morph obtaining as per the quintuplet provided, creating the thumbnail if necessary.  If it is created afresh, it will also be cached at this time"

	| aThumbnail aSymbol formToThumbnail labeledItem |
	aSymbol := aQuint third.
	Thumbnails at: aSymbol ifPresent: [ :thumb | ^thumb ].
	formToThumbnail := aQuint at: 5 ifAbsent: [].
	formToThumbnail ifNil: [
		labeledItem := (Smalltalk at: aQuint first) perform: aQuint second with: aPhidget.
		formToThumbnail := labeledItem imageForm: 32 backgroundColor: aColor forRectangle: labeledItem fullBounds.
		formToThumbnail replaceColor: aColor withColor: Color transparent.
	].

	aThumbnail := Thumbnail new makeThumbnailFromForm: formToThumbnail.
	self cacheThumbnail: aThumbnail forSymbol: aSymbol.
	^ aThumbnail

"PartsBin initialize"! !

!PhdETPartsBin methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 11:47'!
controlingMorph
	^ controlingMorph! !

!PhdETPartsBin methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 11:48'!
controlingMorph: aMorph
	controlingMorph := aMorph.! !

!PhdETPartsBin methodsFor: 'as yet unclassified' stamp: 'pt 5/28/2009 13:20'!
listDirection: aListDirection quadList: quadList
	"Initialize the receiver to run horizontally or vertically, obtaining its elements from the list of tuples of the form:
		(<receiver> <selector> <label> <balloonHelp>)"

	| aButton aClass |
	self layoutPolicy: TableLayout new.
	self listDirection: aListDirection.
	self wrapCentering: #center.
	self layoutInset: 2.
	self cellPositioning: #bottomCenter.

	aListDirection == #leftToRight
		ifTrue:
			[self vResizing: #rigid.
			self hResizing: #spaceFill.
			self wrapDirection: #topToBottom]
		ifFalse:
			[self hResizing: #rigid.
			self vResizing: #spaceFill.
			self wrapDirection: #leftToRight].
	quadList do:
		[:tuple |
			aClass := Smalltalk at: tuple first.
			aButton := IconicButton new initializeWithThumbnail: (self class thumbnailForQuad: tuple color: self color phidget: self phidget) withLabel: tuple third andColor: self color andSend: tuple second to: aClass with: self phidget.
			(tuple size > 3 and: [tuple fourth isEmptyOrNil not]) ifTrue:
				[aButton setBalloonText: tuple fourth].
 			self addMorphBack: aButton]! !

!PhdETPartsBin methodsFor: 'as yet unclassified' stamp: 'pt 5/28/2009 17:11'!
phidget
	^ controlingMorph! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/6/2009 15:55'!
getAttached
	"Answers wether an Interface Kit is attached or not."

	^ self costume getAttached! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/8/2009 08:32'!
getIfKitInputValue
	"Answer the value of an Interface Kits digital input."

	^ self costume getIfKitInputValue! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/8/2009 08:32'!
getIfKitOutputValue
	"Answer the value of an Interface Kits digital output."

	^ self costume getIfKitOutputValue! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/8/2009 08:32'!
getIfKitSensorValue
	"Answer the value of an Interface Kits analog sensor."

	^ self costume getIfKitSensorValue! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/7/2009 19:18'!
getLabel
	"Answers the label of a Phidget"

	^ self costume getLabel! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/7/2009 09:27'!
getPortNumber
	"Answers the index of a Phidget port."

	^ self costume getPortNumber! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 6/4/2009 08:26'!
getServo1MotorCurrentPosition
	"Get the servo motors positio in the intervall [0,10000]."

	^ self costume getCurrentPosition! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 6/4/2009 08:25'!
getServo1MotorCurrentPositionNumber
	"Get the servo motors position."

	^ self costume getCurrentPositionNumber! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 6/3/2009 02:22'!
getServo1MotorEngaged
	"This is wether the motor is engaged or not"

	^ self costume getEngaged! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 6/3/2009 02:20'!
getServo1MotorMaxPosition
	"Get the servo maximum position."
	
	^ self costume getMaxPosition! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 6/3/2009 02:20'!
getServo1MotorMinPosition
	"Get the servo minimum position."
	
	^ self costume getMinPosition! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/7/2009 14:57'!
setIfKitOutputValue: value
	"Set the value of a Phidget Port"

	^ self costume setIfKitOutputValue: value! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/7/2009 19:19'!
setLabel: anObject
	"Sets the label of a Phidget"

	^ self costume setLabel: anObject! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 5/7/2009 09:27'!
setPortNumber: aNumber
	"Answers the index of a Phidget port."

	^ self costume setPortNumber: aNumber! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 6/4/2009 08:26'!
setServo1MotorCurrentPosition: aFloat 
	"Set the servo motors position."
	^ self costume setCurrentPosition: aFloat! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 6/4/2009 08:26'!
setServo1MotorCurrentPositionNumber: aNumber 
	"Set the servo motors position in an Intervall of 0 to 10,000"
	^ self costume setCurrentPositionNumber: aNumber! !

!Player methodsFor: '*Phidget-EToys' stamp: 'pt 6/3/2009 02:46'!
setServo1MotorEngaged: aBoolean 
	"This is wether the motor is engaged or not"
	^ self costume setEngaged: aBoolean! !
